import {ChangeDetectorRef, Component} from '@angular/core'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {RecipeService} from '../../services/recipe.service'
import {Permission} from '../../../shared/model/user'
import {AccountService} from '../../../accounts/services/account.service'
import {getRecipeLuma, Recipe} from '../../../shared/model/recipe.model'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorService} from '../../../shared/service/error.service'
import {AppState} from '../../../shared/app-state'
import {ConfigService} from '../../../shared/service/config.service'
import {Company} from '../../../shared/model/company.model';
import {CompanyService} from '../../../company/service/company.service';
import {Config} from '../../../shared/model/config.model';

@Component({
  selector: 'cre-list',
  templateUrl: './list.component.html',
  styleUrls: ['./list.component.sass']
})
export class ListComponent extends ErrorHandlingComponent {
  companies: Company[] = []
  recipes: Map<number, Recipe[]> = new Map<number, Recipe[]>()
  tableCols = ['name', 'description', 'color', 'sample', 'iconNotApproved', 'buttonView', 'buttonEdit']
  searchQuery = ''
  panelForcedExpanded = false
  hiddenRecipes = []

  constructor(
    private companyService: CompanyService,
    private recipeService: RecipeService,
    private accountService: AccountService,
    private configService: ConfigService,
    private cdRef: ChangeDetectorRef,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()
    this.appState.title = 'Explorateur'

    // Navigate to configs if server is in emergency mode
    this.subscribe(
      this.configService.get(Config.EMERGENCY_MODE),
      config => {
        if (config.content == "true") {
          this.urlUtils.navigateTo('/admin/config/')
        }
      }
    )

    this.fetchCompanies()
    // this.fetchRecipes()
  }

  private fetchCompanies() {
    this.subscribe(
      this.companyService.all,
      companies => this.companies = companies
    )
  }

  private fetchRecipes() {
    this.subscribe(
      this.recipeService.allByCompany,
      recipes => this.recipes = recipes,
      true
    )
  }

  searchRecipes() {
    if (this.searchQuery.length > 0 && !this.panelForcedExpanded) {
      this.panelForcedExpanded = true
      this.cdRef.detectChanges()
    }

    for (let recipeArray of this.recipes.values()) {
      recipeArray.forEach(recipe => this.recipeMatchesSearchQuery(recipe))
    }
  }

  isCompanyHidden(company: Company): boolean {
    const companyRecipes = this.recipes.get(company.id);
    return !(companyRecipes && companyRecipes.length >= 0) ||
      this.searchQuery && this.searchQuery.length > 0 &&
      companyRecipes.map(r => this.hiddenRecipes[r.id]).filter(r => !r).length <= 0
  }

  isLight(recipe: Recipe): boolean {
    return getRecipeLuma(recipe) > 200
  }

  get hasEditPermission(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_RECIPES)
  }

  private recipeMatchesSearchQuery(recipe: Recipe) {
    const matches = this.searchString(recipe.company.name) ||
      this.searchString(recipe.name) ||
      this.searchString(recipe.description) ||
      (recipe.sample && this.searchString(recipe.sample.toString()))
    this.hiddenRecipes[recipe.id] = !matches
  }

  private searchString(value: string): boolean {
    return value.toLowerCase().indexOf(this.searchQuery.toLowerCase()) >= 0
  }
}
